import axios from 'axios';
import { latLng } from 'leaflet';
import LocalGroup from '@/LocalGroup';
import Strike from '@/Strike';
import moment from 'moment';

export default class Api {
  baseUrl: string;

  getAccessToken: () => string | undefined;

  constructor(baseUrl: string, getAccessToken: () => string | undefined) {
    this.baseUrl = baseUrl;
    this.getAccessToken = getAccessToken;
  }

  async localGroups(): Promise<Array<LocalGroup>> {
    return axios.get(`${this.baseUrl}localGroups`)
      .then((response) => response.data.map((json: any) => ({
        coordinates: latLng(json.lat, json.lon),
        name: json.name,
        id: json.id,
      })));
  }

  async strikes(argument: any): Promise<Array<Strike>> {
    return axios.get(`${this.baseUrl}strikes/`, {
      params: argument,
    })
      .then((response) => response.data.map((json: any) => ({
        coordinates: latLng(json.lat, json.lon),
        id: json.id,
        locationName: json.locationName,
        dateTime: moment(json.dateTime),
        eventLink: json.eventLink,
        note: json.note,
      })));
  }

  async searchLocalGroup(name: string): Promise<Array<LocalGroup>> {
    return axios.get(`${this.baseUrl}localGroups`, {
      params: {
        name,
      },
    }).then((response) => response.data.map((json: any) => ({
      coordinates: latLng(json.lat, json.lon),
      id: json.id,
      name: json.name,
    })));
  }

  async createStrike(strike: any): Promise<number> {
    if (!this.getAccessToken()) {
      throw Error('An access_token is needed for createStrike');
    }
    const data = {
      localGroup: strike.localGroupId.toString(),
      organizerNumber: strike.organizerNumber,
      organizerEmail: strike.organizerEmail,
      locationName: strike.locationName,
      locationLon: strike.locationCoordinates.lng.toString(),
      locationLat: strike.locationCoordinates.lat.toString(),
      dateTime: strike.dateTime.unix(),
      eventLink: strike.eventLink ? strike.eventLink : undefined,
      note: strike.note ? strike.note : undefined,
    };
    return axios.put(`${this.baseUrl}strike`, data, {
      headers: {
        Authorization: `Bearer ${this.getAccessToken()}`,
      },
    }).then((response) => ({
      status: response.status,
      ...response.data,
    }));
  }

  async deleteStrike(strike: number) {
    if (!this.getAccessToken()) {
      throw Error('An access_token is needed for deleteStrike');
    }
    return axios.delete(`${this.baseUrl}strike/${strike}`, {
      headers: {
        Authorization: `Bearer ${this.getAccessToken()}`,
      },
    }).then((response) => ({
      status: response.status,
      ...response.data,
    }));
  }
}
