import Api from '@/api';
import AccessToken from '@/AccessToken';
import Vue from 'vue';
import Vuex, { Store } from 'vuex';
import SessionState from '@/SessionState';
import { ToastProgrammatic } from 'buefy';
import App from './App.vue';
import './registerServiceWorker';

import router from './router'; // eslint-disable-line import/no-cycle

Vue.config.productionTip = false;

interface StateInterface {
  api: Api;
  sessionState: SessionState | null;
}

const store: Store<StateInterface> = new Vuex.Store({
  state: {
    api: new Api('https://api.fffutu.re/staging/',
      () => store.state.sessionState?.accessToken.encodedToken),
    sessionState: null,
  } as StateInterface,
  getters: {
    loggedIn(state): boolean {
      return state.sessionState != null;
    },
  },
  mutations: {
    setSessionState(state, sessionState: SessionState) {
      state.sessionState = sessionState;
    },
    removeSessionState(state) {
      state.sessionState = null;
    },
  },
  actions: {
    async setSession({ commit, state }, accessToken: AccessToken) {
      if (state.sessionState) {
        commit('setSessionState', {
          accessToken,
          editableLocalGroups: state.sessionState.editableLocalGroups,
        });
      } else {
        const groupNames = accessToken.groups
          .map((groupName: string) => groupName.replace(/^\/(OG )?/, ''));
        const localGroupPromises = groupNames
          .map((groupName: string) => state.api.searchLocalGroup(groupName));
        Promise.all(localGroupPromises)
          .then((localGroups) => commit('setSessionState', {
            editableLocalGroups: localGroups.flat(),
            accessToken,
          }));
      }
    },
    async endSession({ commit, state }) {
      if (state.sessionState) {
        commit('removeSessionState');
        router.replace('/');
        ToastProgrammatic.open({
          message: 'Du wurdest ausgeloggt',
          type: 'is-info',
          duration: 5000,
        });
      }
    },
  },
});

new Vue({
  router,
  render: (h) => h(App),
  store,
}).$mount('#app');

export default store;
