import Vue from 'vue';
import VueRouter, { RouteConfig, NavigationGuard } from 'vue-router';
import Vuex from 'vuex';
import { ToastProgrammatic as Toast } from 'buefy'; // eslint-disable-line import/no-duplicates
import Buefy from 'buefy'; // eslint-disable-line import/no-duplicates
import { library } from '@fortawesome/fontawesome-svg-core';
// internal icons
import {
  faAngleRight,
  faAngleLeft,
  faExclamationCircle,
  faEnvelope,
  faPhoneAlt,
  faCaretUp,
  faCaretDown,
  faTimes,
  faMapMarkerAlt,
  faClock,
  faInfo,
  faMobileAlt,
  faLock,
} from '@fortawesome/free-solid-svg-icons';
import { FontAwesomeIcon } from '@fortawesome/vue-fontawesome';

import Navbar from '@/components/Navbar.vue';
import Home from '@/views/Home.vue';


library.add(
  faAngleRight,
  faAngleLeft,
  faEnvelope,
  faPhoneAlt,
  faExclamationCircle,
  faCaretUp,
  faCaretDown,
  faTimes,
  faMapMarkerAlt,
  faClock,
  faInfo,
  faMobileAlt,
  faLock,
);
Vue.component('vue-fontawesome', FontAwesomeIcon);

Vue.use(VueRouter);
Vue.use(Vuex);
Vue.use(Buefy, {
  defaultIconComponent: 'vue-fontawesome',
  defaultIconPack: 'fas',
});

Vue.component('navbar', Navbar);


const needLogin: NavigationGuard<Vue> = (_to, _from, next) => {
  const refreshExpire = window.localStorage.getItem('refresh_expires');
  if (!refreshExpire || Number.isNaN(Number(refreshExpire))
    || Number(refreshExpire) < new Date().getTime() / 1000) {
    next({ name: 'Home' });
    Toast.open({
      message: 'Das dürfen Sie nicht! Loggen Sie sich erst einmal ein.',
      type: 'is-danger',
      duration: 5000,
    });
  } else next();
};
const routes: Array<RouteConfig> = [
  {
    path: '/',
    name: 'Home',
    component: Home,
  },
  {
    path: '/about',
    name: 'About',
    // route level code-splitting
    // this generates a separate chunk (about.[hash].js) for this route
    // which is lazy-loaded when the route is visited.
    component: () => import(/* webpackChunkName: "about" */ '../views/About.vue'),
  },
  {
    path: '/strike/manage',
    name: 'ManageStrike',
    props: true,
    component: () => import('../views/ManageStrike.vue'),
    beforeEnter: needLogin,
  },
  {
    path: '/strike/create',
    name: 'Streik erstellen',
    component: () => import('../views/CreateStrike.vue'),
    beforeEnter: needLogin,
  },
];

const router = new VueRouter({
  mode: 'history',
  routes,
});

export default router;
